%This file replicates Figure 1 in Shi (2014)
%
%This program draws the graph to show the location mismatch of the
%asymptotic distribution. In this program, I simulate the distribution of
%the studentized likelihood ratio test for the regression models. The two
%models compared have very different numbers of regressors.  
%
%The first model:
%              Y = a0+a1*X1+...+a8*X_k+epsilon
%The second model:
%              Y = a0+a_{k+1}*X_{k+1}+epsilon
%(X1,...,X_k,epsilon) are independent N(0,1).
%a0=1, a_{k+1}=sqrt(k*a^2), a1=...=a_k=a
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

a = 0.125;     %choose different a to see different levels of mismatch


a0 = 1;    %intercept in the DGP equation


n = 1000;   %sample size
S = 5000; %number of simulation repetitions
d1 = 16;  %number of parameters in model 1
d2 = 2;   %number of parameters in model 2

alpha = 0.05;

rs = RandStream('mt19937ar','Seed',246848906); %set the random number stream.
Tnc_mat = zeros(S,1); %matrix to store the values of the studentized LR

omega_mat = zeros(S,1);
qomega_mat = zeros(S,1);
tic
for s = 1:S;
    %DGP
    X_k = randn(rs,n,d1-1);
    X_kplus1 = randn(rs,n,d2-1);
    epsilon = randn(rs,n,1);    %error term
    
    Y = a0 + a*(sum(X_k,2))/sqrt(d1-1) + a*sum(X_kplus1,2)/sqrt(d2-1) + sqrt(1-a^2)*epsilon;
                                              %generating Y
                                              
    X_I = [ones(n,1),X_k];      %regressors in the first model
    X_II = [ones(n,1),X_kplus1];      %regressors in the second model
    
    %ESTIMATION
    aI_hat = (X_I'*X_I)\(X_I'*Y);
    aII_hat = (X_II'*X_II)\(X_II'*Y);
%     var1_hat = (Y-X_I*aI_hat)'*(Y-X_I*aI_hat)/n;   %variance estimator
%     var2_hat = (Y-X_II*aII_hat)'*(Y-X_II*aII_hat)/n;%variance estimator
%     
    %LR STATISTIC
    LAMDA_hat = -0.5*((Y - X_I*aI_hat).^2) + 0.5*((Y-X_II*aII_hat).^2);
                
            
    LR_hat = mean(LAMDA_hat);
    
    omegasq_hat = mean(LAMDA_hat.^2)-LR_hat^2;
    T_hat = sqrt(n)*LR_hat/sqrt(omegasq_hat);
    
    Tnc_mat(s) = T_hat;   
    
    omega_mat(s) = n*omegasq_hat; 
    
    %TWO-STEP Vuong Test
    res1 = Y-X_I*aI_hat; %residual of the first model
    res2 = Y-X_II*aII_hat; %residual of the second model
    der1_LAMDA_hat = kron(res1,ones(1,d1)).*X_I;
                            %gradient of the first model
    der2_LAMDA_hat = -kron((res2),ones(1,d2)).*X_II;
                            %-gradient of the second model
    B_hat = cov([der1_LAMDA_hat,der2_LAMDA_hat],1);
                            %Covariance of gradients
    AI_hat = -X_I'*X_I/n;
    AII_hat = -X_II'*X_II/n;
    A_hat = [AI_hat,zeros(d1,d2);zeros(d2,d1),-AII_hat];
                            %Hessian of the log-likelihood ratio
    rho_hat = corr([der1_LAMDA_hat,der2_LAMDA_hat],LAMDA_hat);
                            %corrrelation between the gradients and the
                            %log-density ratio.
    
    W_hat = (sqrtm(B_hat)/A_hat)*sqrtm(B_hat);
    [VW,eigW] = eig(W_hat);  %VW*eigW*VW' = W
    D_hatihf = diag(sqrt(1./diag(B_hat)));
    OME_hat = D_hatihf*B_hat*D_hatihf;
    Z0 = randn(rs,S,d1+d2);
    ZW2Z = (Z0.^2)*(diag(eigW^2));
    qomega = quantile(ZW2Z,1-alpha);
    qomega_mat(s) = qomega;  %critical value of the variance test
    
    
    if floor(s/100)*100==s;
    s
    end
end
toc
%%
ncrej  = [mean(abs(Tnc_mat)>norminv(0.975)),mean(omega_mat>qomega_mat),...
                         mean((omega_mat>qomega_mat).*(abs(Tnc_mat)>1.96))]
                     
xi = (-4:0.08:4)';
ftnc = ksdensity(Tnc_mat,xi);      %density of the original Vuong statistic
ftnorm = normpdf(xi);
data = [xi,ftnc,ftnorm];

hfig = figure;
set(hfig,'units','centimeters');
pos = get(hfig,'position');
set(hfig,'position',[pos(1:2),5.5,5.5]);

h0=plot(xi,ftnc,'-',xi,ftnorm,'--','linewidth',2);
axis([-4 4 0 1])
set(gca, 'Box', 'on','xtick',-4:2:4);
% if a==0
% ylabel('density','fontsize',11,'fontweight','b','interpreter','none');
% end
hleg=legend('$\hat{T}_n$','$N(0,1)$');
set(hleg,'location','northwest');
set(hleg,'interpreter','none')
set(hleg,'box','off')


eval(['matlabfrag(''figure1_a_eqs',num2str(a*1000),'.tex'');']);



%%
% eval(['fID = fopen(''fig1_a_eqs_',num2str(a*1000),'.dat'',''w'')']);
% fprintf(fID, 'xi\t nc\t n01 \n');
% fprintf(fID, '%1.4f\t %1.4f\t %1.4f \n',data);
% fclose(fID);
% 
% %eval(['dlmwrite(''fig1_a_eqs_',num2str(a*1000),'.dat'',data,''delimiter'', ''\t'',''precision'',8)']);

