%This is the main program for the simulation Example 3 of Shi (2014). It
%replicates Figure 4.
%
%This program calls for the functions ``ndVuong_cv.m", ``Norm_logfi.m",
%``Norm_loggi.m". The first function computes the critical value of the new
%nondegenerate test. The next two functions define the log-density function
%for the two competing models.
%
%This program also calls for the graphic routine "matlabfrag."
%
%A brief description of Example 3 is as follows:
%
%We consider the following two models:
%
% $F: Y\sim N(theta,1), theta\in R$
% $G: Y\sim N(0,beta), beta\in R_{+}$
%
%The DGP is:
%
% $Y \sim N(\mu, \sigma^2)$;
%
%where $mu = \sqrt(\exp(2*lr-1+\sigma^2)-\sigma^2). Now, $lr =
%E\Lambda_i^{\ast}$, and $\sigma^2$ determines $\omega^2$. 
%
%In this program, I compute the coverage probability of the nondegenerate
%test proposed in Shi (2014), the one-step Vuong test, the two-step test
%and the split-sample test.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
n = 250;    %sample size
S1 = 5000;  %Monte Carlo repetition
S2 = 5001;  %Critical value simulation repetition

alpha = 0.05; %significance level
%%
DGP = 2;      %set this to 1, 2 or 3 for the three sets of DGPs shown in the paper
              %DGP=1: Figure 4(a, b, c)
              %DGP=2: Figure 4(d, e, f)
              %DGP=3: Figure 4(g, h, i)

if DGP == 1;
    sigsq = 0:0.3:6;
    lr = 0*sigsq;
elseif DGP == 2;
    lr = (0:.01:0.2)*(250/n);
    sigsq = 1+0*lr;
elseif DGP == 3;
    lr = (0:0.08:1.6)*sqrt(250/n);
    sigsq = 5+0*lr;
end
%%
tic
rejprob = NaN(length(lr),13);
   %lr~sig~osVg1~osVg2~tsVg1~tsVg2~varT~ndVg1~ndVg2~cstar~split1~split2~n
numpara = length(lr);
for j = 1:numpara;
    lrj = lr(j);
    sigsqj = sigsq(j);
    muj = sqrt(exp(2*lrj-1+sigsqj)-sigsqj);
    
    [sample_rstr,cv_rstr1,cv_rstr2]=RandStream.create('mrg32k3a','NumStreams',3);
    
    T_Vg_vec = NaN(S1,1);    %Test statistic vector for classical Vuong
    VarT_vec = NaN(S1,1);    %Variance test stat in two-step classical Vuong
    T_ndVg_vec = NaN(S1,1);  %Test stat for nondegenerate test
    T_split_vec = NaN(S1,1); %Test stat for split sample test
    cv_VarT = NaN(S1,1);     %critical value for varianc etest in two-step Vg
    cv_ndVg_vec = NaN(S1,1); %critical value for nondegenerate test
    cstar_vec = NaN(S1,1);   %chosen c in nondegenerate test
    for s = 1:S1;
        %Generate the data
        Y0 = randn(sample_rstr,n,1);
        Y = muj + Y0*sqrt(sigsqj);
        
        %Parameter Estimators: (In general, should numerically maximize
        %log-likelihood function. But here the problem is simple enough to have
        %closed form solution. I use closed form solution here.)
        
        theta_hat = mean(Y);
        beta_hat = Y'*Y/n;
        
        %Test preparation
        [logf,d_logf,d2_logf] = Norm_logfi(Y,theta_hat);
        [logg,d_logg,d2_logg] = Norm_loggi(Y,beta_hat);
        
        Zcv = randn(cv_rstr1,S2,2);
        %normal vector used to simulate cv for variance test
        
        nLR_hat = sum(logf-logg);      %sample log-likelihood ratio times n
        
        nomega2_hat = (logf-logg)'*(logf-logg) - nLR_hat^2/n;
        %n times sample variance of \Lambda_i
        A_hat = [sum(d2_logf),0;0,-sum(d2_logg)]/n;
        
        meandL = mean([d_logf,-d_logg]);
        dmeandL = [d_logf,-d_logg] - repmat(meandL,n,1);
        B_hat = dmeandL'*dmeandL/n+10^(-12)*eye(2);
        
        W_hat = (sqrtm(B_hat)/A_hat)*sqrtm(B_hat);
        [~,V_hat]=eig(W_hat);
        
%         dmeanL = logf-logg - nLR_hat/n;
%         cov_hat = dmeandL'*dmeanL/n;     %covariance of Lambda and its first dev
%         rho_star_hat = Q_hat'*pinv(sqrtm(B_hat))*cov_hat/sqrt(nomega2_hat/n);
        
        %Classical Vuong tests
        T_Vg_vec(s) = nLR_hat/sqrt(nomega2_hat); %test stat for classical Vuong
        %(one step and two-step)
        VarT_vec(s) = nomega2_hat;
        cv_VarT(s) = quantile((Zcv.^2)*(diag(V_hat).^2),1-alpha);
        
        
        %Non-degenerate Vuong Tests
        T_ndVg = @(c)(nLR_hat+trace(V_hat)/2)/...
                                   sqrt(nomega2_hat + c*trace(V_hat^2));
        %test stat for the nondegenerate Vuong test
               
        [cv_ndVg_vec(s),cstar_vec(s)]=ndVuong_cv(alpha,V_hat,cv_rstr2,S2);
        
        T_ndVg_vec(s) = T_ndVg(cstar_vec(s));
        
        
        %Split-sample test    (Split the sample into two halves based on the
        %                              natural ordering of the data.)
        sigl1_sq=var(logf);
        sigl2_sq=var(logg);
        covll = corr(logf,logg)*sqrt(sigl1_sq*sigl2_sq);
        
        epsi_n = (max(4*covll/(sigl1_sq+sigl2_sq),0)-1); %robust weight
        
        llnsplit = (1/(2+epsi_n))*(logf(1:n/2,1)-logg((1+n/2):n,1)...
            +(1+epsi_n)*(logf((1+n/2):n,1)-logg(1:n/2,1)));
        LR_split = mean(llnsplit);      %split sample log likelihood ratio
        omegasq_split = var(llnsplit);  %variance of the split sample log density ratio
        T_split = sqrt(n/2)*LR_split/sqrt(omegasq_split);   %split sample test statistic
        T_split_vec(s)=T_split;
        if 20*floor(s/20) == s;
            display([floor([toc,s]),j]);
        end
    end
    
    %Computing the rejection probabilities:
    z_normal = norminv(1-alpha/2);
    rejprob_osVg = [mean(T_Vg_vec>z_normal),mean(T_Vg_vec<-z_normal)];
    rejprob_varT = mean(VarT_vec>cv_VarT);
    rejprob_tsVg = [mean((T_Vg_vec>z_normal).*(VarT_vec>cv_VarT)),...
        mean((T_Vg_vec<-z_normal).*(VarT_vec>cv_VarT))];
    rejprob_ndVg = [mean(T_ndVg_vec>cv_ndVg_vec),mean(T_ndVg_vec<-cv_ndVg_vec)];
    rejprob_split = [mean(T_split_vec>z_normal),mean(T_split_vec<-z_normal)];
    mean_cv_ndVg = mean(cv_ndVg_vec);
    mean_cstar = mean(cstar_vec);
    
    rejprob(j,:)=[lrj,sigsqj,rejprob_osVg,rejprob_tsVg,rejprob_varT,...
                                    rejprob_ndVg,mean_cstar,rejprob_split,n];
    j=j+1;
end;
totaltime = toc;
%Reporting the results:
%% reporting the results

%saving the results in a txt file
eval(['fID=fopen(''Norm_neqs_',num2str(n),'_DGPeqs_',num2str(DGP),'data.txt'',''w'');']);

fprintf(fID,'lrj|sigsqj|osVg1|osVg2|tsVg1|tsVg2|varT|ndVg1|ndVg2|cstar|split1|split2|n\n');
for j=1:length(lr);
    fprintf(fID,...
 '%g|%g|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%g\n',...
                                                           rejprob(j,:));                                                  
end;
fprintf(fID,'_______________________________________________________________\n');
fprintf(fID,'Total Time Elapsed in Minutes = %g',totaltime/60);
fclose(fID);

data = rejprob;
%% Graphing the results
n = data(1,13);
lr = data(:,1);
sigsq = data(:,2);
osVg = sum(data(:,3:4),2);
tsVg = sum(data(:,5:6),2);
ndVg = sum(data(:,8:9),2);
splitVg = sum(data(:,11:12),2);
if max(lr)==0;
    DGP=1;
elseif max(sigsq) == 1;
    DGP=2;
else
    DGP=3;
end

hfig = figure;
set(hfig,'units','centimeters');
pos = get(hfig,'position');
set(hfig,'position',[pos(1:2),5.5,5.5]);
hold on;
if DGP==1;
    
    h0=plot(sigsq,osVg,'-.',sigsq,tsVg,'--',...
        sigsq,splitVg,':.',sigsq,ndVg,'-','linewidth',2);
    axis([0 6 0 0.15])
    plot(sigsq,0.05+0*sigsq,':','linewidth',1);
    set(gca, 'Box', 'on','xtick',0:1:6,'ytick',0:0.05:0.15);
    xlabel('$\sigma^2$','fontsize',11,'fontweight','b','interpreter','none');
else
    h0=plot(lr,osVg,'-.',lr,tsVg,'--',...
        lr,splitVg,':.',lr,ndVg,'-','linewidth',2);
    plot(lr,0.05+0*lr,':',lr,0.5+0*lr,'-','linewidth',1);
    xlabel('$lr$','fontsize',11,'fontweight','b','interpreter','none');
    if DGP==2;
        axis([0 0.2*(250/n) 0 1]);
        set(gca, 'Box', 'on','xtick',(0:.05:.2)*(250/n),'ytick',0:0.25:1);
    else
        axis([0 1.6*sqrt(250/n) 0 1]);
        set(gca, 'Box', 'on','xtick',round((0:.4:1.6)*sqrt(250/n)*10)/10,'ytick',0:0.25:1);
    end
end

eval(['matlabfrag(''Norm_neqs_',num2str(n),'_DGPeqs_',num2str(DGP),'.tex'');']);


