%This is the main program for the simulation Example 1 of Shi (2014). It
%replicates the numbers in Table 1. To obtain all numbers in the table,
%adjust "a1", "a2", "d_F", "d_G" and "n" below. 
%
%This main program calls the functions "logfi" and "loggi" which compute
%the log-density, first derivative, and second derivative of the two models
%respectively.  
%
%A brief description:
%
%In this program, we consider the following two models:
%
% F: Y = theta_1 + Z1*theta_2 + u, u\sim N(0,sigma_f^2)
% G: Y = beta_1 + Z2*beta_2 + v, v\sim N(0,sigma_g^2)
%
%The DGP is:
%
% Y = 1 + (a1/sqrt(d_F-2))*sum(Z1) + (a2/sqrt(d_G-2))*sum(Z2) + e
%
%     where
%        (Z1,Z2,e)\sim N(0,I)
%
%    When a1=a2, the H_0 holds. When a1>a2, model F is better. The
%    omega_P^2 depends on both the magnitude of a1 and that of a2.
%
%In this program, I compute the coverage probability of the nondegenerate
%test proposed in Shi (2014), the one-step Vuong test, the two-step test
%and the split-sample test.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear

n = 100;    %sample size

a1 = 0.25;
a2 = 0.25;

%a1 = sqrt(1.09^(250/n)-1);
%a2 = 0;

d_F = 3; %dimension of model F  (d_F = K_f+2 for K_f in the paper)
d_G = 6;%dimension of model G  (d_G = K_g+2 for K_g in the paper)

k = d_F+d_G;

S1 = 5000;  %Monte Carlo repetition
S2 = 5001;  %Critical value simulation repetition

alpha = 0.05; %significance level



[sample_rstr,cv_rstr1,cv_rstr2]=RandStream.create('mrg32k3a','NumStreams',3);

T_Vg_vec = NaN(S1,1);    %Test statistic vector for classical Vuong
VarT_vec = NaN(S1,1);    %Variance test stat in two-step classical Vuong
T_ndVg_vec = NaN(S1,1);  %Test stat for nondegenerate test
T_split_vec = NaN(S1,1); %Test stat for split sample test
cv_VarT = NaN(S1,1);     %critical value for varianc etest in two-step Vg
cv_ndVg_vec = NaN(S1,1); %critical value for nondegenerate test
cstar_vec = NaN(S1,1);   %chosen c in nondegenerate test
tic;
for s = 1:S1;
    %Generate the data
    Z1 = randn(sample_rstr,n,d_F-2);
    Z2 = randn(sample_rstr,n,d_G-2);
    er = randn(sample_rstr,n,1);
    Y = 1+ (a1/sqrt(d_F-2))*sum(Z1,2)+(a2/sqrt(d_G-2))*sum(Z2,2)+er;
    
    X = [Y,Z1,Z2];  %The whole data matrix put together
    
    %Parameter Estimators: (In general, should numerically maximize
    %log-likelihood function. But here the problem is simple enough to have
    %closed form solution. I use closed form solution here.)
    Z1cons = [ones(n,1),Z1];
    Z2cons = [ones(n,1),Z2];
    
    theta_hat = (Z1cons'*Z1cons)\(Z1cons'*Y);       %coefficient estimators
    beta_hat = (Z2cons'*Z2cons)\(Z2cons'*Y);
    varf_hat = (Y-Z1cons*theta_hat)'*(Y-Z1cons*theta_hat)/n; %variance estimator
    varg_hat = (Y-Z2cons*beta_hat)'*(Y-Z2cons*beta_hat)/n;
    
    %Test preparation
    [logf,d_logf,d2_logf] = logfi(X,[theta_hat;varf_hat]);
    [logg,d_logg,d2_logg] = loggi(X,[beta_hat;varg_hat]);
    Zcv = randn(cv_rstr1,S2,k); 
                       %normal vector used to simulate cv for variance test
    
    nLR_hat = sum(logf-logg);         %sample log-likelihood ratio times n
    
    nomega2_hat = (logf-logg)'*(logf-logg) - nLR_hat^2/n;
                                      %n times sample variance of \Lambda_i
    A_hat = [reshape(sum(d2_logf),d_F,d_F),zeros(d_F,d_G);zeros(d_G,d_F),...
                                           -reshape(sum(d2_logg),d_G,d_G)]/n;
    
    meandL = mean([d_logf,-d_logg]);
    dmeandL = [d_logf,-d_logg] - repmat(meandL,n,1);
    B_hat = dmeandL'*dmeandL/n;
    
    W_hat = (sqrtm(B_hat)/A_hat)*sqrtm(B_hat);
    [Q_hat,V_hat]=eig(W_hat);
    
%     dmeanL = logf-logg - nLR_hat/n;
%     cov_hat = dmeandL'*dmeanL/n;     %covariance of Lambda and its first dev
%     rho_star_hat = Q_hat'*pinv(sqrtm(B_hat))*cov_hat/sqrt(nomega2_hat/n);
    
    %Classical Vuong tests
    
    T_Vg_vec(s) = nLR_hat/sqrt(nomega2_hat); %test stat for classical Vuong
                                          %(one step and two-step)
    VarT_vec(s) = nomega2_hat;
    cv_VarT(s) = quantile((Zcv.^2)*(diag(V_hat).^2),1-alpha);
    

    %Non-degenerate Vuong Tests    
    T_ndVg = @(c)(nLR_hat+trace(V_hat)/2)/...
                              sqrt(nomega2_hat + c*trace(V_hat.^2));
                                %test stat for the nondegenerate Vuong test
    
    [cv_ndVg_vec(s),cstar_vec(s)]=ndVuong_cv(alpha,V_hat, cv_rstr2,S2);
    T_ndVg_vec(s) = T_ndVg(cstar_vec(s));
    
    
    %Split-sample test    (Split the sample into two halves based on the
    %                              natural ordering of the data.)
    sigl1_sq=var(logf);
    sigl2_sq=var(logg);
    covll = corr(logf,logg)*sqrt(sigl1_sq*sigl2_sq);
    
    epsi_n = (max(4*covll/(sigl1_sq+sigl2_sq),0)-1);
    
    llnsplit = (1/(2+epsi_n))*(logf(1:n/2,1)-logg((1+n/2):n,1)...
                             +(1+epsi_n)*(logf((1+n/2):n,1)-logg(1:n/2,1)));
    LR_split = mean(llnsplit);      %split sample log likelihood ratio
    omegasq_split = var(llnsplit);  %variance of the split sample log density ratio
    T_split = sqrt(n/2)*LR_split/sqrt(omegasq_split);   %split sample test statistic
    T_split_vec(s)=T_split;
    if 100*floor(s/100) == s;
    display(floor([toc,s]));
    end
end
%%
%Computing the rejection probabilities:
z_normal = norminv(1-alpha/2);
rejprob_osVg = [mean(T_Vg_vec>z_normal),mean(T_Vg_vec<-z_normal)];
rejprob_varT = mean(VarT_vec>cv_VarT);
rejprob_tsVg = [mean((T_Vg_vec>z_normal).*(VarT_vec>cv_VarT)),...
                    mean((T_Vg_vec<-z_normal).*(VarT_vec>cv_VarT))];
rejprob_ndVg = [mean(T_ndVg_vec>cv_ndVg_vec),mean(T_ndVg_vec<-cv_ndVg_vec)];
rejprob_split = [mean(T_split_vec>z_normal),mean(T_split_vec<-z_normal)];
mean_cstar = mean(cstar_vec);

%Reporting the results:
%% reporting the results

eval(['fID=fopen(''NorReg_neqs_',num2str(n),'_a1eqs_',num2str(a1),'_a2eqs_',num2str(a2),'.txt'',''w'');']);
fprintf(fID,'Results from NorReg_Base.m\n');
fprintf(fID,'_________________________________________________\n');
fprintf(fID, 'n = %g, S1 = %g, S2 = %g, d_F = %g, d_G = %g\n',[n,S1,S2,d_F,d_G]);
fprintf(fID,'(Nominal Size = %g)\n',alpha);
fprintf(fID,'__________________________________________________\n');
fprintf(fID,'                    Null Rejection probabilities: \n');
fprintf(fID,'                    (a1 = %g, a2 = %g)\n\n',[a1,a2]);
fprintf(fID,'2Step. Test              [%1.4f,%1.4f] \n',rejprob_tsVg);
fprintf(fID,'1STep. Test              [%1.4f,%1.4f] \n',rejprob_osVg);
fprintf(fID,'Var. Test Rej. Prob = %1.4f\n',rejprob_varT);
fprintf(fID,'Nondegenerate. Test      [%1.4f,%1.4f] \n',rejprob_ndVg);
fprintf(fID,'Average cstar of nd Test = %1.4f\n',mean_cstar);
fprintf(fID,'Split-Sample Test        [%1.4f,%1.4f] \n',rejprob_split);

totaltime=toc;
fprintf(fID,'__________________________________________________\n');
fprintf(fID,'Total time elapsed in minutes = %1.4f\n',totaltime/60);
fclose(fID);



