%This is the critical value function, that computes the simulated critical
%value for the nondegenerate Vuong test in Shi (2013). The computation
%follows the procedure described on pages 14-15 of the paper to choose c.
%In addition, it also only search over $\sigma\in[0,\infty]$ and use the
%plug-in estimateor for $\rho^\ast$. 
%
%The function takes as input:
%
% alpha:     significance level. in (0,1).
% V:         this is eig(B^{1/2}A^{-1}B^{1/2}). A diagonal matrix.
% rho_star:  this is Q'pinv(B^{1/2})*D^{1/2}*rho. A column vector, same
%            dimension as each row of V.
% rs:        random stream
% S_cv:      number of simulation draws used to simulate the quantile
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [cv,cstar] = ndVuong_cv(alpha,V,rs,S_cv)

k = length(V(:,1));
vecV = diag(V);
abs_vecV = abs(vecV)-max(abs(vecV));
rho_star = 1*(abs_vecV==0);
rnorm = rho_star'*rho_star;
rho_star = sqrt(rnorm)^(-1)*rho_star;

Z0 = randn(rs,S_cv,k+1);

VZ = [1,rho_star';rho_star,eye(k)];

Z = Z0*sqrtm(VZ+10^(-12)*eye(k+1));

Z_L = Z(:,1);            %$Z_Lambda$
Z_p = Z(:,2:k+1);        %$Z_phi^\ast$

trVsq = trace(V^2);
Vnmlzd = V/sqrt(trVsq);   %V, normalized by sqrt(trVsq);

J_Lmod = @(sig,c)sig*Z_L - (Z_p.^2)*diag(Vnmlzd)/2+ trace(Vnmlzd)/2;

J_omod = @(sig,c)sig^2 - 2*sig*Z_p*Vnmlzd*rho_star + ...
                                       (Z_p.^2)*(diag(Vnmlzd).^2) + c;

quant = @(sig,c)quantile(abs(J_Lmod(sig,c)./sqrt(J_omod(sig,c))),1-alpha);

                                  % quantile as a function of sigma and c.
               
sigstar = @(c)fminbnd(@(sig)-quant(sig,c),0,5);

cv0 = quant(sigstar(0),0);        % critical value with c=0

z_normal = norminv(1-alpha/2);
z_nor_sim = max(z_normal,quantile(abs(Z_L),1-alpha)); %simulated z_normal

if cv0 - z_nor_sim <= 0.1;  % if critical value with c=0 is not very big
 
    cv = max(cv0,z_normal);                     % use c = 0
    
    cstar = 0;
    
else                              % otherwise, increase c.

    Func = @(c)((quant(sigstar(c),c)-z_nor_sim)-0.1)^2;
    cstar = fminbnd(Func,0,10);
    cv = max(quant(sigstar(cstar),cstar),z_normal);
end

















