/***Mata part for implementing the nondegenerate vuong test and the classical one-step Vuong test***/

/*This is the second part of the replication files of the empirical application of Shi (2014), and should be run after running after running the first part: CoateConlinModels_Estimation.do. 

Part of this file is the STATA routine for the nondegenerate Vuong test that is generically applicable to the comparison of any other pairs of models that are estimated by the "ml" routine built-in STATA. The generic part of the code is indicated below.*/

/*The code calls for the quantile function mm_quantile() from the "moremata" package,  the moremata package is available here: http://ideas.repec.org/c/boc/bocode/s455001.html*/

/*This include code for three pairwise tests. Choose below after getting information from Stata*/

/*When the code finishes running, the last line of the log file contains the test results: 
(alpha, cstar, cvnd, Tnd, Tclassical)
alpha: significance level
cstar: cstar defined in the paper
cvnd: critical value of nondegenerate test
Tnd:  test statistic of nondegenerate test
Tclassical: test statistic of classical Vuong test*/

/*To find p-value of the nondegenerate test, change the significance level below. Try different values to find the smallest one that makes the nondegenerate test statistic exceed the critical value. Typically, no more than 10 tries are needed for 3 digit accuracy. An informal algorithm is as follows:
  (1) first try alpha = 0.05
  (2) if Tnd>cvnd, decrease alpha. If Tnd>cvnd by a lot, decrease alpha more, if a little, decrease alpha less
  (3) repeat. 
  
  The p-value of the classical Vuong test is 2*(1-normcdf(Tclassical)) */

capture log close
log using ndVuong_int_rf.log,append

mata          //start mata session
mata clear

S_cv=10000   //number of simulation draws used to simulate the critical value

alpha=0.073  //significance level (try different values to find p-value)

/*Get Score, Hessian and log-density Information from the ml estimation run in Stata:*/

logf_group=st_data(.,"group_lnf")                 //log-density of the three models
logf_sur=st_data(.,"sur_lnf")
logf_intensity = st_data(.,"intensity_lnf")

grad_group=st_data(.,"$gradlist_group")            //gradient of log-density -- the scores of the three models
grad_sur = st_data(.,"$gradlist_sur")
grad_intensity = st_data(.,"$gradlist_intensity")

group_Hinv=st_matrix("group_Hinv")                 //Hessian of log-likelihood  of the three models
intensity_Hinv=st_matrix("intensity_Hinv")
sur_Hinv=st_matrix("sur_Hinv")

/*Choose to uncomment one of the following three paragraphs to choose the pair being compared. 
  When one paragraph is uncommented, the other two should be commented*/

/*
 //Uncomment this paragraph if comparing group rule-utilitarian with the reduced form model
grad = (grad_group,(-grad_sur))
logf = logf_group
logg = logf_sur
Afinv = group_Hinv
Aginv = sur_Hinv*/

/*
//Uncomment this paragraph if comparing the group rule-utilitarian with the intensity model
grad = (grad_group,(-grad_intensity))
logf = logf_group
logg = logf_intensity
Afinv = group_Hinv
Aginv = intensity_Hinv*/


//Uncomment this paragraph if comparing the intensity model with the reduced form model
grad = (grad_intensity,(-grad_sur))
logf = logf_intensity
logg = logf_sur
Afinv = intensity_Hinv
Aginv = sur_Hinv



/* ***********************************************************************************************************
*
*Generic STATA (MATA) routine for the nondegenerate Vuong test
*
*This routine takes the five matrices "grad", "logf", "logg", "Afinv" and "Aginv" as input. The five
*input matrices are:
*
*  grad: a n*(d_theta+d_beta) matrix, where the first d_theta columns are the gradient of the first 
         model, and the last d_beta coluns are minus the gradient of the second model. The gradients 
         can be obtained by the "ml score" command after "ml" estimation.
*  logf: a n*1 vector that is the log-density of the first model.
*  logg: a n*1 vector that is the log_density of the second model.
* Afinv: a d_\theta*d_\theta matrix that is the inverse of the second derivative of the log-likelihood
*        function for the first model. This equals -e(V) where e(V) is a system macro produced by the "ml"
*        estimation routine in Stata.
* Aginv: a d_beta*d_beta matrix that is the inverse of the second derivative of the log-likelihood
*        function for the second model. This equals -e(V) where e(V) is a system macro produced by the "ml"
*        estimation routine in Stata.
*
*This routine produces a five dimensional vector "cvs" as output. The five coordinates of "cvs" are 
*respectively:
*
*cvs[1]=significance level   //chosen by the user
*cvs[2]=cstar                //the data dependent c* in the nondegenerate Vuong test
*cvs[3]=cv                   //critical value of the nondegenerate Vuong test
*cvs[rows(cvs),4]=Tnd        //test statistic of the nondegenerate Vuong test
*cvs[rows(cvs),5]=T          //test statistic of the one-step classical Vuong (1989) test
************************************************************************************************************ */


k=cols(grad)

nsample = rows(grad)


/*Computing B matrix and the inverse of A matrix*/
B=crossdev(grad,mean(grad),grad,mean(grad))/rows(grad)	
ZeroMat = J(rows(Afinv),cols(Aginv),0)
Ainv=Afinv*nsample,ZeroMat\ZeroMat',-Aginv*nsample	

/*Computing V matrix*/
symeigensystem(B,X=.,L=.)
sqrtB=X*diag(sqrt(abs(L)))*X'
symeigensystem(sqrtB*Ainv*sqrtB,X=.,vecV=.)
vecV=vecV'
V=diag(vecV)


abs_vecV=abs(vecV):-max(abs(vecV))
rho_star=(abs_vecV:==0)
rnorm=cross(rho_star,rho_star)
rho_star=sqrt(rnorm)^(-1)*rho_star

trVsq=trace(V:^2)
Vnmlzd=V/sqrt(trVsq)

real scalar quant(real scalar sig,real scalar c)
{
	real colvector temp_J_Lmod
	real colvector temp_J_omod
	external real Z_L
	external real Z_p
	external real Vnmlzd
	external real rho_star
	external alpha
	temp_J_Lmod=sig*Z_L-(Z_p:^2)*diagonal(Vnmlzd)/2:+trace(Vnmlzd)/2
	temp_J_omod=sig^2:-2*sig*Z_p*Vnmlzd*rho_star:+(Z_p:^2)*(diagonal(Vnmlzd):^2):+c	
	return(mm_quantile(abs(temp_J_Lmod:/sqrt(temp_J_omod)),1,1-alpha))
}
void eval(todo,s,c,v,g,H)
{
	v=quant(exp(s),c)
}
real scalar sigstar(real scalar c)
{
	real scalar temp_sigstar
	external real alpha
	S=optimize_init()
	optimize_init_evaluator(S,&eval())
	optimize_init_params(S,0)
	optimize_init_technique(S,"nm")
	optimize_init_tracelevel(S,"none")
	optimize_init_nmsimplexdeltas(S, 1)
	optimize_init_argument(S,1,c)
	temp_sigstar=optimize(S)
	return(exp(temp_sigstar))
}
/*Parameter initialization*/
cvs = J(1,5,-99)

/*Classical Vuong test statistic*/
nLR_hat=sum(logf-logg)
nomega2_hat=sum((logf-logg:-mean(logf-logg)):^2)
T=nLR_hat/sqrt(nomega2_hat)


/*Compute test statistic, cstar and cv at 5% level (result stored in the last row of cvs)*/
rseed(13579)
Z0=rnormal(S_cv,k+1,0,1)
VZUpper=(1,rho_star'\J(k,1,0),I(k))
VZsqrt = VZUpper - diag(0\rho_star)

Z=Z0*VZsqrt
Z_L=Z[.,1]
Z_p=Z[.,2..k+1]
cv0=quant(sigstar(0),0)

z_normal=max((mm_quantile(abs(Z_L),1,1-alpha/2),invnormal(1-alpha/2)))
z_normal_l = min((mm_quantile(abs(Z_L),1,1-alpha/2),invnormal(1-alpha/2)))

if (cv0-z_normal<=0.1) {
	cv = z_normal
	cstar=0
}
else {
    csl = 0
	csu = 1
	qcl = cv0
	qcu = quant(sigstar(csu),csu)
	while (qcu>z_normal+0.09) {
	    csl = csl+1
	    csu = csu+1   
	    qcl = qcu
	    qcu = quant(sigstar(csu),csu)
		qcu
		csu
	}
	cstar = csl + (z_normal+0.1-qcl)/(qcu-qcl)
	qcstar = quant(sigstar(cstar),cstar)
	while (min((qcstar-z_normal_l-0.09,z_normal+0.1-qcstar))<-0.0001) {
	    if (qcstar-z_normal-0.09<0) {
		    csu = cstar
			qcu = qcstar
			cstar = csl + (csu-csl)*(z_normal+0.1-qcl)/(qcu-qcl)
			sigstarc = sigstar(cstar)
			qcstar = quant(sigstarc,cstar)
			sigstarc
			(csu,cstar,csl\qcu,qcstar,qcl)
		}
		if (z_normal+0.1-qcstar<0) {
		    csl = cstar
			qcl = qcstar
			cstar  = csl + (csu-csl)*(z_normal+0.1-qcl)/(qcu-qcl)
			sigstarc = sigstar(cstar)
			qcstar = quant(sigstarc,cstar)
			sigstarc
			(csu,cstar,csl\qcu,qcstar,qcl)
			
		}
	}
	cv=qcstar
}
/*test statistic*/
Tnd=(nLR_hat+trace(V)/2)/sqrt(nomega2_hat+cstar*trace(V:^2))

cvs[rows(cvs),1]=alpha
cvs[rows(cvs),2]=cstar
cvs[rows(cvs),3]=cv
cvs[rows(cvs),4]=Tnd
cvs[rows(cvs),5]=T

/*alpha, cstar, cvnd, Tnd, Tclassical*/
cvs

end

log close
